
const addrInput = document.getElementById("vcfAddr")
const nameInput = document.getElementById("vcfName")
const keyInput = document.getElementById("vcfKey")
const sendButton = document.getElementById("vcfSend")
const errorElem = document.getElementById("error")

function showError(msg) {
    errorElem.style.display = "block"
    errorElem.innerText = msg
}

function nameToFilename(name) {
    return name
        .normalize('NFKD') // Normalize Unicode characters
        .trim()
        .replace(/[\/\\:*?"<>|]+/g, '')      // Remove unsafe filename characters
        .replace(/\s+/g, '-')                // Replace whitespace with dashes
        .replace(/-+/g, '-')                 // Replace multiple dashes with a single dash
        .toLowerCase()
        .concat('.vcf');
}


/* .asc key have the format:

-----BEGIN PGP PUBLIC KEY BLOCK-----
Optional Header: Foo
More optional header lines before key starts after empty line: Bar

mQINBE7u/F4BEADNkng1S+8C/llxQ7DP+pTr9DkYHt+e2zz2WthTgcRxZD7dPut1
T2i/5gr7BYuOqrJZVc7L5BwMt+xr6J+jovhbtvC6bpIC61WpCB66vrDgFb9lfb8g
wKPAjvxsnei6SytUYPSYGzuuTofh7Zjc2J/uimhuEYz1BC0Nu9tIenyxCy9433rZ
6hUYxw6Xkt9+5MtMK9J0M771v3tnYO05E+WfQSJGbXiKEqTZOw==
=eY5D
-----END PGP PUBLIC KEY BLOCK-----

*/
function extractBase64FromAsc(ascString) {
    const lines = ascString.split(/\r?\n|\r/);

    let inBlock = false;
    let base64Lines = [];

    for (let line of lines) {
        line = line.trim();

        // Detect the start and end of the PGP block
        if (line.startsWith('-----BEGIN PGP')) {
            inBlock = true;
            continue;
        }
        if (line.startsWith('-----END PGP')) {
            break;
        }

        if (inBlock) {
            // Skip headers and the optional checksum (starts with '=')
            if (
              line === '' ||
              line.includes(': ') ||       // skip headers
              line.startsWith('=')         // skip checksum line
            ) {
              continue;
            }
            base64Lines.push(line);
        }
    }

    return base64Lines.join('');
}

function sendVCard(e) {
    var addr = addrInput.value.trim()
    if (addr == "") {
        showError("Please enter an e-mail address below")
        return
    }
    if (!addr.includes("@")) {
        showError("E-mail lacks character @")
        return
    }

    var name = nameInput.value
    if (name == "") {
        name = addr
    }

    const filename = nameToFilename(name)

    var key = keyInput.value.trim()
    if (key == "") {
        showError("Please set a key")
        return
    }
    if (!key.includes("-----BEGIN PGP PUBLIC KEY BLOCK-----")) {
        showError("Key lacks marker -----BEGIN PGP PUBLIC KEY BLOCK-----")
        return
    }
    if (!key.includes("-----END PGP PUBLIC KEY BLOCK-----")) {
        showError("Key lacks marker -----END PGP PUBLIC KEY BLOCK-----")
        return
    }
    key = extractBase64FromAsc(key)

    // timestamp needs the format YYYYMMDDTHHMMSSZ
    const timestamp = new Date().toISOString().replace(/[-:]/g, '').replace(/\.\d+Z$/, 'Z');

    const data = "BEGIN:VCARD\n"
        + "VERSION:4.0\n"
        + "EMAIL:" + addr + "\n"
        + "FN:" + name + "\n"
        + "KEY:data:application/pgp-keys;base64," + key + "\n"
        + "REV:" + timestamp + "\n"
        + "END:VCARD"

    window.webxdc.sendToChat({file: {name: filename, plainText: data}})
}


sendButton.addEventListener("click", sendVCard)


document.getElementById("uploadBtn").addEventListener("click", function() {
    document.getElementById("fileInput").click();
});


document.getElementById("fileInput").addEventListener("change", function(event) {
    const file = event.target.files[0]
    if (file) {
        const reader = new FileReader()
        reader.onload = function() {
            keyInput.value = reader.result;
        }
        reader.readAsText(file)
    }
});